export interface IPluginInitializedMessageData {
    readonly projectId: string | null;
    readonly languageCodename: string | null;
    readonly highlighterEnabled: boolean;
}
export interface IElementClickMessageData {
    readonly projectId: string;
    readonly languageCodename: string;
    readonly itemId: string;
    readonly elementCodename: string;
}
export interface IHighlightsStatusMessageData {
    readonly enabled: boolean;
}
export declare enum IFrameMessageType {
    Initialized = "kontent:initialized",
    ElementClicked = "kontent:element:click",
    HighlightsStatus = "kontent:highlights:status"
}
export declare type IFrameMessagesMap = {
    readonly [IFrameMessageType.Initialized]: (data: IPluginInitializedMessageData) => void;
    readonly [IFrameMessageType.ElementClicked]: (data: IElementClickMessageData) => void;
    readonly [IFrameMessageType.HighlightsStatus]: (data: IHighlightsStatusMessageData) => void;
};
export interface IFrameMessage<E extends keyof IFrameMessagesMap> {
    readonly type: E;
    readonly data: Parameters<IFrameMessagesMap[E]>[0];
}
export declare class IFrameCommunicator {
    private readonly events;
    constructor();
    addMessageListener: <M extends IFrameMessageType>(type: M, listener: IFrameMessagesMap[M]) => void;
    removeMessageListener: <M extends IFrameMessageType>(type: M, listener: IFrameMessagesMap[M]) => void;
    destroy(): void;
    sendMessage: <M extends IFrameMessageType>(type: M, data: Parameters<IFrameMessagesMap[M]>[0]) => void;
    private onMessage;
}
