"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var iframe_1 = require("./utils/iframe");
var link_1 = require("./utils/link");
var IFrameCommunicator_1 = require("./lib/IFrameCommunicator");
var NodeSmartLinkProvider_1 = require("./lib/NodeSmartLinkProvider");
var storage_1 = require("./utils/storage");
var validation_1 = require("./utils/validation");
var query_1 = require("./utils/query");
var initialState = {
    queryTimerId: 0,
};
var defaultConfiguration = {
    languageCodename: null,
    projectId: null,
    queryParam: 'kontent-smart-link-enabled',
};
var Plugin = /** @class */ (function () {
    function Plugin(configuration) {
        var _this = this;
        this.destroy = function () {
            var _a;
            _this.unwatchQueryParams();
            _this.nodeSmartLinkProvider.destroy();
            (_a = _this.iFrameCommunicator) === null || _a === void 0 ? void 0 : _a.destroy();
        };
        this.updateConfiguration = function (configuration) {
            _this.configuration = __assign(__assign({}, _this.configuration), configuration);
        };
        this.updateState = function (state) {
            _this.state = __assign(__assign({}, _this.state), state);
        };
        this.initializeIFrameCommunication = function () {
            if (!iframe_1.isInsideIFrame() || _this.iFrameCommunicator) {
                throw new Error('Cannot initialize iframe communication while not inside the iframe.');
            }
            _this.iFrameCommunicator = new IFrameCommunicator_1.IFrameCommunicator();
            var storage = storage_1.createStorage('kontent:plugin:iframe-settings');
            var stored = storage.get();
            var highlighterEnabled = stored !== null ? stored === null || stored === void 0 ? void 0 : stored.highlighterEnabled : true;
            if (highlighterEnabled) {
                _this.nodeSmartLinkProvider.enable();
            }
            _this.iFrameCommunicator.addMessageListener(IFrameCommunicator_1.IFrameMessageType.HighlightsStatus, function (data) {
                if (!data || !_this.nodeSmartLinkProvider)
                    return;
                _this.toggleNodeSmartLinkProvider(data.enabled);
                storage.set({
                    highlighterEnabled: data.enabled,
                });
            });
            _this.iFrameCommunicator.sendMessage(IFrameCommunicator_1.IFrameMessageType.Initialized, {
                projectId: _this.configuration.projectId,
                languageCodename: _this.configuration.languageCodename,
                highlighterEnabled: highlighterEnabled,
            });
        };
        this.watchQueryParams = function () {
            if (iframe_1.isInsideIFrame()) {
                throw new Error('No need to watch query params inside the iframe.');
            }
            if (_this.state.queryTimerId) {
                clearTimeout(_this.state.queryTimerId);
            }
            var isPluginEnabled = query_1.isQueryParamPresent(_this.configuration.queryParam);
            var hasChanged = _this.nodeSmartLinkProvider.enabled !== isPluginEnabled;
            if (hasChanged) {
                _this.toggleNodeSmartLinkProvider(isPluginEnabled);
            }
            _this.updateState({
                queryTimerId: setTimeout(_this.watchQueryParams, 1000),
            });
        };
        this.unwatchQueryParams = function () {
            clearTimeout(_this.state.queryTimerId);
        };
        this.listenToHighlighterEvents = function () {
            _this.nodeSmartLinkProvider.addEventListener(NodeSmartLinkProvider_1.NodeSmartLinkProviderEventType.ElementClicked, _this.onElementClick);
        };
        this.onElementClick = function (elementClickData) {
            var data = __assign(__assign({}, elementClickData), { projectId: elementClickData.projectId || _this.configuration.projectId || undefined, languageCodename: elementClickData.languageCodename || _this.configuration.languageCodename || undefined });
            if (validation_1.validateElementClickMessageData(data)) {
                if (iframe_1.isInsideIFrame() && _this.iFrameCommunicator) {
                    _this.iFrameCommunicator.sendMessage(IFrameCommunicator_1.IFrameMessageType.ElementClicked, data);
                }
                else {
                    var link = link_1.buildElementLink(data.projectId, data.languageCodename, data.itemId, data.elementCodename);
                    window.open(link, '_blank');
                }
            }
        };
        this.toggleNodeSmartLinkProvider = function (force) {
            var shouldEnable = typeof force !== 'undefined' ? force : !_this.nodeSmartLinkProvider.enabled;
            if (shouldEnable) {
                _this.nodeSmartLinkProvider.enable();
            }
            else {
                _this.nodeSmartLinkProvider.disable();
            }
        };
        this.nodeSmartLinkProvider = new NodeSmartLinkProvider_1.NodeSmartLinkProvider();
        this.state = initialState;
        this.configuration = __assign(__assign({}, defaultConfiguration), configuration);
        if (iframe_1.isInsideIFrame()) {
            this.initializeIFrameCommunication();
        }
        else {
            this.watchQueryParams();
        }
        this.listenToHighlighterEvents();
    }
    return Plugin;
}());
var PluginWrapper = /** @class */ (function () {
    function PluginWrapper() {
    }
    PluginWrapper.initializeOnLoad = function (configuration) {
        return new Promise(function (resolve) {
            window.addEventListener('load', function () {
                resolve(PluginWrapper.initialize(configuration));
            });
        });
    };
    PluginWrapper.initialize = function (configuration) {
        if (!PluginWrapper.plugin) {
            PluginWrapper.plugin = new Plugin(configuration);
        }
        return PluginWrapper.plugin;
    };
    PluginWrapper.prototype.destroy = function () {
        var _a;
        (_a = PluginWrapper.plugin) === null || _a === void 0 ? void 0 : _a.destroy();
        PluginWrapper.plugin = null;
    };
    PluginWrapper.prototype.setConfiguration = function (configuration) {
        if (!PluginWrapper.plugin) {
            throw new Error('KontentSmartLink is not initialized or has already been destroyed.');
        }
        else {
            PluginWrapper.plugin.updateConfiguration(configuration);
        }
    };
    return PluginWrapper;
}());
exports.default = PluginWrapper;
//# sourceMappingURL=plugin.js.map